/*****************************************************************************************************************************
--
--    File Name    : cdcfifo.v

--    Description  : This module implements CDC fifo                                 

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module cdcfifo #(parameter[3 : 0] ADDR_WIDTH = 12,
                 parameter[7 : 0] DATA_WIDTH = 8)
               (k_lim,
                wrrstn,
                wrclk,
                wrreq,
                wrdata,
                wrfull,
                wrusedw,
                rdrstn,
                rdclk,
                rdreq,
                rddata,
                rdempty,
                rdusedw);
  input[ADDR_WIDTH - 1 : 0] k_lim;
  input wrrstn;
  input wrclk;
  input wrreq;
  input[(DATA_WIDTH - 1) : 0] wrdata;
  output reg wrfull;
  output[(ADDR_WIDTH - 1) : 0] wrusedw;
  input rdrstn;
  input rdclk;
  input rdreq;
  output[(DATA_WIDTH - 1) : 0] rddata;
  output rdempty;
  output reg[(ADDR_WIDTH - 1) : 0] rdusedw;
  parameter[31 : 0] lOI = 32'h00000001;
  parameter[31 : 0] oOI = 32'h00000003;
  wire iOI;
  wire OII;
  wire III;
  reg[(ADDR_WIDTH - 1) : 0] lII;
  reg[(ADDR_WIDTH - 1) : 0] oII;
  reg[(ADDR_WIDTH - 1) : 0] iII;
  reg[(ADDR_WIDTH - 1) : 0] OlI;
  reg[(ADDR_WIDTH - 1) : 0] IlI;
  reg[(ADDR_WIDTH - 1) : 0] llI;
  reg[(ADDR_WIDTH - 1) : 0] olI;
  reg[(ADDR_WIDTH - 1) : 0] ilI;
  reg[(ADDR_WIDTH - 1) : 0] O0I;
  reg[(ADDR_WIDTH - 1) : 0] I0I;
  reg[(ADDR_WIDTH - 1) : 0] l0I;
  reg[(ADDR_WIDTH - 1) : 0] o0I;
  wire[(ADDR_WIDTH - 2) : 0] i0I;
  reg O1I,
      I1I;
  assign i0I = { ADDR_WIDTH - 1 { 1'b0 } };
  cdcfiforam #(.ADDR_WIDTH(ADDR_WIDTH), .DATA_WIDTH(DATA_WIDTH)) l1I(.data(wrdata),
                                                                     .wren(OII),
                                                                     .wraddress(IlI),
                                                                     .rdaddress(lII),
                                                                     .wrclock(wrclk),
                                                                     .rdclock(rdclk),
                                                                     .q(rddata));
  always
    @(olI or
      i0I)
    begin: o1I
      reg[(ADDR_WIDTH - 1) : 0] i1I;
      reg OoI;
      integer IoI;
      integer loI;
      i1I[(ADDR_WIDTH - 1)] = olI[(ADDR_WIDTH - 1)];
      i1I[(ADDR_WIDTH - 2) : 0] = i0I;
      OoI = 1'b0;
      for (IoI = 0; IoI <= (ADDR_WIDTH - 1); IoI = IoI + 1)
        begin
          for (loI = IoI; loI <= (ADDR_WIDTH - 1); loI = loI + 1)
            i1I[IoI] = i1I[IoI] ^ olI[loI];
          O0I[IoI] <= olI[IoI] ^ ((~OoI) & (~i1I[IoI]));
          OoI = OoI | (~i1I[IoI]);
        end
    end
  always
    @(negedge wrrstn or
      posedge wrclk)
    begin: ooI
      reg[(ADDR_WIDTH - 1) : 0] ioI;
      reg[(ADDR_WIDTH - 1) : 0] OiI;
      integer IoI;
      if (wrrstn == 1'b0)
        begin
          IlI <= { ADDR_WIDTH { 1'b0 } };
          llI <= { ADDR_WIDTH { 1'b0 } };
          ilI <= lOI[(ADDR_WIDTH - 1) : 0];
          olI <= oOI[(ADDR_WIDTH - 1) : 0];
          iII <= { ADDR_WIDTH { 1'b0 } };
          OlI <= { ADDR_WIDTH { 1'b0 } };
          o0I <= { ADDR_WIDTH { 1'b0 } };
          wrfull <= 1'b0;
          OiI = { ADDR_WIDTH { 1'b0 } };
          ioI = { ADDR_WIDTH { 1'b0 } };
        end
      else
        begin
          if (OII == 1'b1)
            begin
              olI <= O0I;
              ilI <= olI;
              IlI <= ilI;
            end
          llI <= IlI;
          iII <= lII;
          OlI <= iII;
          ioI[(ADDR_WIDTH - 1)] = llI[(ADDR_WIDTH - 1)];
          for (IoI = (ADDR_WIDTH - 2); IoI >= 0; IoI = IoI - 1)
            ioI[IoI] = ioI[IoI + 1] ^ llI[IoI];
          OiI[(ADDR_WIDTH - 1)] = OlI[(ADDR_WIDTH - 1)];
          for (IoI = (ADDR_WIDTH - 2); IoI >= 0; IoI = IoI - 1)
            OiI[IoI] = OiI[IoI + 1] ^ OlI[IoI];
          if (o0I >= k_lim)
            wrfull <= 1'b1;
          else
            wrfull <= 1'b0;
          { I1I, o0I } <= ioI - OiI;
        end
    end
  assign wrusedw = o0I;
  assign OII = wrreq;
  always
    @(lII or
      iOI or
      i0I)
    begin: IiI
      reg[(ADDR_WIDTH - 1) : 0] i1I;
      reg OoI;
      integer IoI;
      integer loI;
      i1I[(ADDR_WIDTH - 1)] = lII[(ADDR_WIDTH - 1)];
      i1I[(ADDR_WIDTH - 2) : 0] = i0I;
      OoI = 1'b0;
      for (IoI = 0; IoI <= (ADDR_WIDTH - 1); IoI = IoI + 1)
        begin
          for (loI = IoI; loI <= (ADDR_WIDTH - 1); loI = loI + 1)
            i1I[IoI] = i1I[IoI] ^ lII[loI];
          oII[IoI] <= lII[IoI] ^ (iOI & (~OoI) & (~i1I[IoI]));
          OoI = OoI | (~i1I[IoI]);
        end
    end
  always
    @(negedge rdrstn or
      posedge rdclk)
    begin: liI
      reg[(ADDR_WIDTH - 1) : 0] oiI;
      reg[(ADDR_WIDTH - 1) : 0] iiI;
      integer IoI;
      if (rdrstn == 1'b0)
        begin
          lII <= { ADDR_WIDTH { 1'b0 } };
          I0I <= { ADDR_WIDTH { 1'b0 } };
          l0I <= { ADDR_WIDTH { 1'b0 } };
          rdusedw <= { ADDR_WIDTH { 1'b0 } };
          oiI = { ADDR_WIDTH { 1'b0 } };
          iiI = { ADDR_WIDTH { 1'b0 } };
        end
      else
        begin
          lII <= oII;
          I0I <= llI;
          l0I <= I0I;
          iiI[(ADDR_WIDTH - 1)] = l0I[(ADDR_WIDTH - 1)];
          for (IoI = (ADDR_WIDTH - 2); IoI >= 0; IoI = IoI - 1)
            iiI[IoI] = iiI[IoI + 1] ^ l0I[IoI];
          oiI[(ADDR_WIDTH - 1)] = lII[(ADDR_WIDTH - 1)];
          for (IoI = (ADDR_WIDTH - 2); IoI >= 0; IoI = IoI - 1)
            oiI[IoI] = oiI[IoI + 1] ^ lII[IoI];
          { O1I, rdusedw } <= iiI - oiI;
        end
    end
  assign III = (l0I == lII) ? 1'b1 : 1'b0;
  assign rdempty = III;
  assign iOI = rdreq & (~III);
endmodule
